function paral_retinex(img_colr_name,Amax,img_ref_name,choix_spec,...
                       sig1,sig2,sig3,...
                       MSR_colr_name,MSR_gmap_name,...
                       SSR_1r_colr_name,SSR_1r_gmap_name,...
                       SSR_1r2_colr_name,SSR_1r2_gmap_name)


img_colr    = double(imread(img_colr_name));    %                 [0,255]
img_gray    = sum(img_colr,3)/3 + 1;            % WARNING: img in [1,256]

Amax        = str2double(Amax);
choix_spec  = str2double(choix_spec);
sig1        = str2double(sig1);
sig2        = str2double(sig2);
sig3        = str2double(sig3);

%%% Parameters %%%
sigma       = [sig1,sig2,sig3];                 % scales

%%% Initialisations %%%
N           = length(sigma);
[H,W]       = size(img_gray);
[Gh,Gw]     = meshgrid( -pi:(pi/W):+pi-(pi/W), -pi:(pi/H):+pi-(pi/H) );
MAX_gmap    = min(255./max(img_colr,[],3),Amax);% the NaNs become Amax
msr_n       = zeros(H,W);
img_p       = cat(1,img_gray,img_gray(end:-1:1,:));     % img 2x bigger
img_p       = cat(2,img_p,img_p(:,end:-1:1));           % img 4x bigger
img_t       = fft2(img_p);                              % FT{img} (2x[H,W])

for n=1:N
    % Fourrier Gaussian Filtering %                     %------------------                       
    gauss = exp(-(sigma(n)^2)*(Gh.^2 + Gw.^2)/2);       % FT{gaussian_sig}
    img_f = abs(ifft2(img_t.*fftshift(gauss)));         % gauss. filtering
    img_f = img_f(1:H,1:W);                             % crop >> img_filt
    
    % MSR %
    msr_n = msr_n + (log(img_gray) - log(img_f))/N;
end

% MSR gain map computation %
msr_n           = ipolSCB(msr_n,.5,.5,255);             % .5% of saturation
MSR_gmap        = min(max(msr_n./(img_gray-1),0),MAX_gmap);  % No color sat

MSR_colr        = zeros(size(img_colr));
MSR_colr(:,:,1) = img_colr(:,:,1).*MSR_gmap;
MSR_colr(:,:,2) = img_colr(:,:,2).*MSR_gmap;
MSR_colr(:,:,3) = img_colr(:,:,3).*MSR_gmap;

% MSR with specification on DxO result
% REF_gray        = sum(double(imread(img_ref_name)),3)/3;
% msr_n_spec      = 255*specification(msr_n/255,REF_gray/255,512);
% MSR_spec_gmap   = max(msr_n_spec./(img_gray-1),0);      % no limitations
% MSR_spec_gmap(MSR_spec_gmap >= 50 ) = 0;
% MSR_spec_colr   = zeros(size(img_colr));
% MSR_spec_colr(:,:,1) = img_colr(:,:,1).*MSR_spec_gmap;
% MSR_spec_colr(:,:,2) = img_colr(:,:,2).*MSR_spec_gmap;
% MSR_spec_colr(:,:,3) = img_colr(:,:,3).*MSR_spec_gmap;

% MSR specification:
REF = double(imread(img_ref_name));
if choix_spec == 1
    msr_n_spec = 255*specification(msr_n/255,sum(REF,3)/(3*255));
    MSR_spec_gmap = max(msr_n_spec./(img_gray-1),0);
    MSR_spec_gmap(MSR_spec_gmap >= 50 ) = 0;
    MSR_spec_colr(:,:,1) = img_colr(:,:,1).*MSR_spec_gmap;
    MSR_spec_colr(:,:,2) = img_colr(:,:,2).*MSR_spec_gmap;
    MSR_spec_colr(:,:,3) = img_colr(:,:,3).*MSR_spec_gmap;
else
    MSR_spec_colr(:,:,1) = 255*specification(MSR_colr(:,:,1)/255,REF(:,:,1)/255);
    MSR_spec_colr(:,:,2) = 255*specification(MSR_colr(:,:,2)/255,REF(:,:,2)/255);
    MSR_spec_colr(:,:,3) = 255*specification(MSR_colr(:,:,3)/255,REF(:,:,3)/255);
end

imwrite(uint8(MSR_colr),MSR_colr_name);                 % Color Image
imwrite(uint8(MSR_gmap*255/Amax),MSR_gmap_name);        % Gray Gain Map
imwrite(uint8(MSR_spec_colr),['spec_' MSR_colr_name]);  % Col Specified Img



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SSR with kernel 1/r

clear MSR_colr MSR_gmap
clear gauss img_f msr_n
clear Gh Gw

[Gh2,Gw2]   = meshgrid( -W:1:+(W-1), -H:1:+(H-1) );
ker_1r      = 1./sqrt((Gh2).^2 + (Gw2).^2);
ker_1r(H+1,W+1) = 1;
ker_1r2     = ker_1r.^2;
ker_1r      = ker_1r./sum(ker_1r(:));
ker_1r2     = ker_1r2./sum(ker_1r2(:));

% Fourrier Gaussian Filtering %                     %------------------                       
ker_1rF = abs(fft2(ker_1r));                        % FT{kernel}
ker_1r2F= abs(fft2(ker_1r2));                       % FT{kernel}
img_flt = abs(ifft2(img_t.*ker_1rF));               % filtering
img_flt2= abs(ifft2(img_t.*ker_1r2F));              % filtering
img_flt = img_flt(1:H,1:W);                         % crop >> img_filt
img_flt2= img_flt2(1:H,1:W);                        % crop >> img_filt

SSR_1r  = log(img_gray) - log(img_flt);
SSR_1r2 = log(img_gray) - log(img_flt2) ;


% Contrast coefficients computation
SSR_1r      = ipolSCB(SSR_1r,.5,.5,255);
SSR_1r_gmap = min(max(SSR_1r./(img_gray-1),0),MAX_gmap);% For no saturation

SSR_1r2     = ipolSCB(SSR_1r2,.5,.5,255);
SSR_1r2_gmap= min(max(SSR_1r2./(img_gray-1),0),MAX_gmap);% For no saturation

% Applying the contrast to each color channel
% 1/r
SSR_1r_colr(:,:,1) = img_colr(:,:,1).*SSR_1r_gmap;
SSR_1r_colr(:,:,2) = img_colr(:,:,2).*SSR_1r_gmap;
SSR_1r_colr(:,:,3) = img_colr(:,:,3).*SSR_1r_gmap;

% 1/r^2
SSR_1r2_colr(:,:,1) = img_colr(:,:,1).*SSR_1r2_gmap;
SSR_1r2_colr(:,:,2) = img_colr(:,:,2).*SSR_1r2_gmap;
SSR_1r2_colr(:,:,3) = img_colr(:,:,3).*SSR_1r2_gmap;

% Specification SSR 1/r and 1/r^2 on DxO result
% -------------
REF = double(imread(img_ref_name));
if choix_spec == 1
    
    SSR_1r_spec     = 255*specification(SSR_1r /255,sum(REF,3)/(3*255));
    SSR_1r2_spec    = 255*specification(SSR_1r2/255,sum(REF,3)/(3*255));
    
    SSR_1r_spec_gmap        = max( SSR_1r_spec./(img_gray-1),0);
    SSR_1r2_spec_gmap       = max(SSR_1r2_spec./(img_gray-1),0);
    SSR_1r_spec_gmap( SSR_1r_spec_gmap  >= 50 ) = 0;
    SSR_1r2_spec_gmap(SSR_1r2_spec_gmap >= 50 ) = 0;
    
    SSR_1r_spec_colr(:,:,1)  = img_colr(:,:,1).*SSR_1r_spec_gmap;
    SSR_1r_spec_colr(:,:,2)  = img_colr(:,:,2).*SSR_1r_spec_gmap;
    SSR_1r_spec_colr(:,:,3)  = img_colr(:,:,3).*SSR_1r_spec_gmap;
    
    SSR_1r2_spec_colr(:,:,1) = img_colr(:,:,1).*SSR_1r2_spec_gmap;
    SSR_1r2_spec_colr(:,:,2) = img_colr(:,:,2).*SSR_1r2_spec_gmap;
    SSR_1r2_spec_colr(:,:,3) = img_colr(:,:,3).*SSR_1r2_spec_gmap;
else
    
    SSR_1r_spec_colr        = zeros(size(img_colr));
    SSR_1r_spec_colr(:,:,1) = 255*specification(SSR_1r_colr(:,:,1)/255,REF(:,:,1)/255);
    SSR_1r_spec_colr(:,:,2) = 255*specification(SSR_1r_colr(:,:,2)/255,REF(:,:,2)/255);
    SSR_1r_spec_colr(:,:,3) = 255*specification(SSR_1r_colr(:,:,3)/255,REF(:,:,3)/255);
    
    SSR_1r2_spec_colr       = zeros(size(img_colr));
    SSR_1r2_spec_colr(:,:,1)= 255*specification(SSR_1r2_colr(:,:,1)/255,REF(:,:,1)/255);
    SSR_1r2_spec_colr(:,:,2)= 255*specification(SSR_1r2_colr(:,:,2)/255,REF(:,:,2)/255);
    SSR_1r2_spec_colr(:,:,3)= 255*specification(SSR_1r2_colr(:,:,3)/255,REF(:,:,3)/255);
end

% Saving images

imwrite(uint8(SSR_1r_colr),SSR_1r_colr_name);               % Color Image
imwrite(uint8(SSR_1r_gmap*255/Amax),SSR_1r_gmap_name);      % Gray Gain Map
imwrite(uint8(SSR_1r_spec_colr),['spec_' SSR_1r_colr_name]);

imwrite(uint8(SSR_1r2_colr),SSR_1r2_colr_name);             % Color Image
imwrite(uint8(SSR_1r2_gmap*255/Amax),SSR_1r2_gmap_name);    % Gray Gain Map
imwrite(uint8(SSR_1r2_spec_colr),['spec_' SSR_1r2_colr_name]);

end
