function out = ipolSCB(in,s1,s2,Dyn)
% ipolSCB stand for IPOL Simplest Color Balance.
% 
% out = ipolSCB(in, s1, s2 [,Dyn])
% 
% with : - in   : input image, gray level.
%        - s1   : percentage of saturation for the white
%        - s2   : percentage of saturation for the black
%        - Dyn  : optional : max value in the output image (default: 255)
%        - out  : output image, in [0,Dyn];
%
% REFERENCE :
%   Simplest Color Balance, Nicolas Limare, Jose-Luis Lisani, Jean-Michel
%   Morel, Ana Beln Petro, and Catalina Sbert
%   Image Processing On Line, 1 (2011).
%   http://dx.doi.org/10.5201/ipol.2011.llmps-scb
%
% Written by : C. Hessel -- September 28, 2015


if nargin == 3, Dyn = 255; end;

[H,W]       = size(in);
N           = H*W;

in_sorted   = sort(in(:));
vmax        = in_sorted(round(N-s1*N/100));
vmin        = in_sorted(round(1+s2*N/100));

if vmax <= vmin
    out     = vmax*ones(H,W);
else
    out     = (in-vmin).*(Dyn/(vmax-vmin));
    out(in < vmin)  = 0;
    out(in > vmax)  = Dyn;
end

fprintf('--- IPOL Simplest Color Balance ---\n');
fprintf('Nb of white clipped pixels: %2.2f %%\n',sum(in(:)>vmax)*100/N);
fprintf('Nb of black clipped pixels: %2.2f %%\n',sum(in(:)<vmin)*100/N);
fprintf('-----------------------------------\n');

end